      SUBROUTINE PDSYGV5( IBTYPE, UPLO, N, A, IA, JA, DESCA,
     $                    B, IB, JB, DESCB, IL, IU, ABSTOL, M, W,
     $                    WORK, LWORK, IWORK, LIWORK, IFAIL, INFO )
*
* Scalar Arguments ..
      CHARACTER          UPLO
      INTEGER            IA, IB, IBTYPE, INFO, JA, JB, IL, IU,
     $                   LIWORK, LWORK, M, N, IFAIL
      DOUBLE PRECISION   ABSTOL
*
* Array Arguments ..
*
      INTEGER            DESCA( * ), DESCB( * ), IWORK( * )
      DOUBLE PRECISION   A( * ), B( * ), W( * ), WORK( * )
*
* Parameters ..
      INTEGER            BLOCK_CYCLIC_2D, DLEN_, DTYPE_, CTXT_, M_, N_,
     $                   MB_, NB_, RSRC_, CSRC_, LLD_
      PARAMETER          ( BLOCK_CYCLIC_2D = 1, DLEN_ = 9, DTYPE_ = 1,
     $                   CTXT_ = 2, M_ = 3, N_ = 4, MB_ = 5, NB_ = 6,
     $                   RSRC_ = 7, CSRC_ = 8, LLD_ = 9 )
      DOUBLE PRECISION   ONE
      PARAMETER          ( ONE = 1.0D+0 )
      DOUBLE PRECISION   FIVE, ZERO
      PARAMETER          ( FIVE = 5.0D+0, ZERO = 0.0D+0 )
      INTEGER            IERRNPD
      PARAMETER          ( IERRNPD = 16 )
*
* Local Scalars ..
      LOGICAL            LQUERY, UPPER
      CHARACTER          TRANS
      INTEGER            ANB, IACOL, IAROW, IBCOL, IBROW, ICOFFA,
     $                   ICOFFB, ICTXT, IROFFA, IROFFB, LIWMIN, LWMIN,
     $                   LWOPT, MQ0, MYCOL, MYROW, NB, NEIG, NN, NP0,
     $                   NPCOL, NPROW, NPS, NQ0, NSYGST_LWOPT,
     $                   NSYTRD_LWOPT, SQNPC
      DOUBLE PRECISION   EPS, SCALE
*
* Local Arrays ..
      INTEGER            IDUM1( 3 ), IDUM2( 3 )
*
* External Functions ..
      LOGICAL            LSAME
      INTEGER            ICEIL, INDXG2P, NUMROC, PJLAENV
      DOUBLE PRECISION   PDLAMCH
      EXTERNAL           LSAME, ICEIL, INDXG2P, NUMROC, PJLAENV, PDLAMCH
*
* External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, CHK1MAT, DGEBR2D, DGEBS2D,
     $                   DSCAL, PCHK2MAT, PDPOTRF, PDSYEV5,
     $                   PDSYNGST, PXERBLA
*
* Intrinsic Functions ..
      INTRINSIC          ABS, DBLE, ICHAR, INT, MAX, MIN, MOD, SQRT
*
* Executable Statements ..
*
* Get grid parameters
*
      ICTXT = DESCA( CTXT_ )
      CALL BLACS_GRIDINFO( ICTXT, NPROW, NPCOL, MYROW, MYCOL )
*
* Test the input parameters
*
      INFO = 0
      IF( NPROW.EQ.-1 ) THEN
         INFO = -( 700+CTXT_ )
      ELSE IF( DESCA( CTXT_ ).NE.DESCB( CTXT_ ) ) THEN
         INFO = -( 1100+CTXT_ )
      ELSE
*
* Get machine constants.
*
         EPS = PDLAMCH( DESCA( CTXT_ ), 'Precision' )
*
         UPPER = LSAME( UPLO, 'U' )
         CALL CHK1MAT( N, 3, N, 3, IA, JA, DESCA, 7, INFO )
         CALL CHK1MAT( N, 3, N, 3, IB, JB, DESCB, 11, INFO )
         IF( INFO.EQ.0 ) THEN
            IF( MYROW.EQ.0 .AND. MYCOL.EQ.0 ) THEN
               WORK( 1 ) = ABSTOL
               WORK( 2 ) = ZERO
               WORK( 3 ) = ZERO
*
               CALL DGEBS2D( DESCA( CTXT_ ), 'ALL', ' ', 3, 1, WORK, 3 )
            ELSE
               CALL DGEBR2D( DESCA( CTXT_ ), 'ALL', ' ', 3, 1, WORK, 3,
     $                       0, 0 )
            END IF
            IAROW = INDXG2P( IA, DESCA( MB_ ), MYROW, DESCA( RSRC_ ),
     $              NPROW )
            IBROW = INDXG2P( IB, DESCB( MB_ ), MYROW, DESCB( RSRC_ ),
     $              NPROW )
            IACOL = INDXG2P( JA, DESCA( NB_ ), MYCOL, DESCA( CSRC_ ),
     $              NPCOL )
            IBCOL = INDXG2P( JB, DESCB( NB_ ), MYCOL, DESCB( CSRC_ ),
     $              NPCOL )
            IROFFA = MOD( IA-1, DESCA( MB_ ) )
            ICOFFA = MOD( JA-1, DESCA( NB_ ) )
            IROFFB = MOD( IB-1, DESCB( MB_ ) )
            ICOFFB = MOD( JB-1, DESCB( NB_ ) )
*
* Compute the total amount of space needed
*
            LQUERY = .FALSE.
            IF( LWORK.EQ.-1 .OR. LIWORK.EQ.-1 )
     $         LQUERY = .TRUE.
*
            LIWMIN = 6*MAX( N, ( NPROW*NPCOL )+1, 4 )
*
            NB = DESCA( MB_ )
            NN = MAX( N, NB, 2 )
            NP0 = NUMROC( NN, NB, 0, 0, NPROW )
*
               LWMIN = 5*N + MAX( 5*NN, NB*( NP0+1 ) )
               LWOPT = LWMIN
               NEIG = 0
*
* Compute how much workspace is needed to use the
* new TRD and GST algorithms
*
            ANB = PJLAENV( ICTXT, 3, 'PDSYTTRD', 'L', 0, 0, 0, 0 )
            SQNPC = INT( SQRT( DBLE( NPROW*NPCOL ) ) )
            NPS = MAX( NUMROC( N, 1, 0, 0, SQNPC ), 2*ANB )
            NSYTRD_LWOPT = 2*( ANB+1 )*( 4*NPS+2 ) + ( NPS+4 )*NPS
            NB = DESCA( MB_ )
            NP0 = NUMROC( N, NB, 0, 0, NPROW )
            NQ0 = NUMROC( N, NB, 0, 0, NPCOL )
            NSYGST_LWOPT = 2*NP0*NB + NQ0*NB + NB*NB
            LWOPT = MAX( LWOPT, N+NSYTRD_LWOPT, NSYGST_LWOPT )
*
* Version 1.0 Limitations
*
            IF( IBTYPE.LT.1 .OR. IBTYPE.GT.3 ) THEN
               INFO = -1
            ELSE IF( .NOT.UPPER .AND. .NOT.LSAME( UPLO, 'L' ) ) THEN
               INFO = -2
            ELSE IF( N.LT.0 ) THEN
               INFO = -3
            ELSE IF( IROFFA.NE.0 ) THEN
               INFO = -5
            ELSE IF( ICOFFA.NE.0 ) THEN
               INFO = -6
            ELSE IF( DESCA( MB_ ).NE.DESCA( NB_ ) ) THEN
               INFO = -( 700+NB_ )
            ELSE IF( DESCA( M_ ).NE.DESCB( M_ ) ) THEN
               INFO = -( 1100+M_ )
            ELSE IF( DESCA( N_ ).NE.DESCB( N_ ) ) THEN
               INFO = -( 1100+N_ )
            ELSE IF( DESCA( MB_ ).NE.DESCB( MB_ ) ) THEN
               INFO = -( 1100+MB_ )
            ELSE IF( DESCA( NB_ ).NE.DESCB( NB_ ) ) THEN
               INFO = -( 1100+NB_ )
            ELSE IF( DESCA( RSRC_ ).NE.DESCB( RSRC_ ) ) THEN
               INFO = -( 1100+RSRC_ )
            ELSE IF( DESCA( CSRC_ ).NE.DESCB( CSRC_ ) ) THEN
               INFO = -( 1100+CSRC_ )
            ELSE IF( DESCA( CTXT_ ).NE.DESCB( CTXT_ ) ) THEN
               INFO = -( 1100+CTXT_ )
            ELSE IF( IROFFB.NE.0 .OR. IBROW.NE.IAROW ) THEN
               INFO = -9
            ELSE IF( ICOFFB.NE.0 .OR. IBCOL.NE.IACOL ) THEN
               INFO = -10
            ELSE IF( IL.LT.1 .OR. IL.GT.MAX( 1, N ) ) THEN
               INFO = -12
            ELSE IF( IU.LT.MIN( N, IL ) .OR. IU.GT.N ) THEN
               INFO = -13
            ELSE IF( ABS( WORK( 1 )-ABSTOL ).GT.FIVE*EPS*ABS( ABSTOL ))
     $                THEN
               INFO = -14
            ELSE IF( LWORK.LT.LWMIN .AND. .NOT.LQUERY ) THEN
               INFO = -18
            ELSE IF( LIWORK.LT.LIWMIN .AND. .NOT.LQUERY ) THEN
               INFO = -20
            END IF
         END IF
         IDUM1( 1 ) = IBTYPE
         IDUM2( 1 ) = 1
         IF( UPPER ) THEN
            IDUM1( 2 ) = ICHAR( 'U' )
         ELSE
            IDUM1( 2 ) = ICHAR( 'L' )
         END IF
         IDUM2( 2 ) = 2
         IF( LQUERY ) THEN
            IDUM1( 3 ) = -1
         ELSE
            IDUM1( 3 ) = 1
         END IF
         IDUM2( 3 ) = 3
         CALL PCHK2MAT( N, 3, N, 3, IA, JA, DESCA, 7, N, 3, N, 3, IB,
     $                  JB, DESCB, 11, 3, IDUM1, IDUM2, INFO )
      END IF
*
      IWORK( 1 ) = LIWMIN
      WORK( 1 ) = DBLE( LWOPT )
*
      IF( INFO.NE.0 ) THEN
         CALL PXERBLA( ICTXT, 'PDSYGV5 ', -INFO )
         RETURN
      ELSE IF( LQUERY ) THEN
         RETURN
      END IF
*
* Form a Cholesky factorization of sub( B ).
*
      CALL PDPOTRF( UPLO, N, B, IB, JB, DESCB, INFO )
      IF( INFO.NE.0 ) THEN
         IWORK( 1 ) = LIWMIN
         WORK( 1 ) = DBLE( LWOPT )
         IFAIL = INFO
         INFO = IERRNPD
         RETURN
      END IF
*
* Transform problem to standard eigenvalue problem and solve.
*
      CALL PDSYNGST( IBTYPE, UPLO, N, A, IA, JA, DESCA, B, IB, JB,
     $               DESCB, SCALE, WORK, LWORK, INFO )
      CALL PDSYEV5( UPLO, N, A, IA, JA, DESCA, IL, IU, ABSTOL, M, W,
     $              WORK, LWORK, IWORK, LIWORK, INFO )
*
      IF( SCALE.NE.ONE ) THEN
         CALL DSCAL( N, SCALE, W, 1 )
      END IF
*
      IWORK( 1 ) = LIWMIN
      WORK( 1 ) = DBLE( LWOPT )
      RETURN
*
*     End of PDSYGV5
*
      END
